/*
 *=BEGIN SONGBIRD GPL
 *
 * This file is part of the Songbird web player.
 *
 * Copyright(c) 2005-2011 POTI, Inc.
 * http://www.songbirdnest.com
 *
 * This file may be licensed under the terms of of the
 * GNU General Public License Version 2 (the ``GPL'').
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the GPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the GPL along with this
 * program. If not, go to http://www.gnu.org/licenses/gpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 *=END SONGBIRD GPL
 */

#include "sbIDevice.h"
#include "sbIMediaItem.h"

#include "nsStringAPI.h"


/**
 * A class for adjusting the properties of media items when imported
 * from a device.  Rules for which media items to adjust and how are
 * specified by <import> elements in device info XML files.  Those
 * rules are resolved in device add-ons (e.g., the MTP add-on) before
 * invoking this class.
 */
class sbDeviceMediaItemDecorator
{
public:
  /**
   * Add or modify properties of aMediaItem according to aImportType
   *
   * \param aDevice       The device on which the media item content resides
   *
   * \param aMediaItem    A media item to decorate according to aImportType
   *
   * \param aImportType   A symbolic name that determines how to decorate
   *                      aMediaItem, such as what properties to add and how
   *                      to compute their values.  Recognized types are:
   *                        fm-recording
   *                        voice-recording
   */
  static nsresult DecorateMediaItem(sbIDevice * aDevice,
                                    sbIMediaItem * aMediaItem,
                                    const nsAString & aImportType);
};
